// src/hf-transformersjs-completion-language-model-openai-compatible.ts
import { pipeline, TextStreamer } from "@huggingface/transformers";
var HFTransformersjsCompletionLanguageModelOpenAICompatible = class {
  constructor(modelId, settings, config) {
    this.specificationVersion = "v1";
    this.defaultObjectGenerationMode = "json";
    this.modelId = modelId;
    this.settings = settings;
    this.config = config;
    this.provider = config.provider;
  }
  async ensurePipeline() {
    if (!this.pipelineInstance) {
      this.pipelineInstance = await pipeline("text-generation", this.modelId, {});
    }
    return this.pipelineInstance;
  }
  doGenerate(options) {
    return this.doGenerateImpl(options);
  }
  async doGenerateImpl(options) {
    var _a, _b, _c, _d;
    const pn = await this.ensurePipeline();
    const promptText = this.getPromptText(options.prompt);
    const generationOptions = {
      max_new_tokens: (_a = options.maxTokens) != null ? _a : 512,
      do_sample: ((_b = options.temperature) != null ? _b : 0) > 0,
      temperature: (_c = options.temperature) != null ? _c : 0
    };
    const res = await pn(promptText, generationOptions);
    const result = Array.isArray(res) ? res : [res];
    const promptTokens = Math.ceil(promptText.length / 4);
    const generatedText = (_d = result[0][-1].generated_text.toString()) != null ? _d : "";
    const completionTokens = Math.ceil(generatedText.length / 4);
    return {
      text: generatedText,
      finishReason: "stop",
      usage: {
        promptTokens,
        completionTokens
      },
      rawCall: {
        rawPrompt: promptText,
        rawSettings: generationOptions
      },
      rawResponse: {
        body: result
      },
      request: {
        body: JSON.stringify({ prompt: promptText, ...generationOptions })
      }
    };
  }
  doStream(options) {
    return this.doStreamImpl(options);
  }
  async doStreamImpl(options) {
    var _a, _b, _c;
    const pn = await this.ensurePipeline();
    const promptText = this.getPromptText(options.prompt);
    const generationOptions = {
      max_new_tokens: (_a = options.maxTokens) != null ? _a : 512,
      do_sample: ((_b = options.temperature) != null ? _b : 0) > 0,
      temperature: (_c = options.temperature) != null ? _c : 0
    };
    const stream = new ReadableStream({
      start: async (controller) => {
        try {
          const tokenCallback = (token) => {
            if (token) {
              controller.enqueue({
                type: "text-delta",
                textDelta: token
              });
            }
          };
          const streamer = new TextStreamer(pn.tokenizer, {
            skip_prompt: true,
            callback_function: tokenCallback
          });
          const streamerOptions = {
            ...generationOptions,
            streamer
          };
          const result = await pn(promptText, streamerOptions);
          controller.enqueue({
            type: "finish",
            finishReason: "stop"
          });
          controller.close();
          return result;
        } catch (error) {
          controller.error(error);
        }
      }
    });
    return {
      stream,
      rawCall: {
        rawPrompt: promptText,
        rawSettings: generationOptions
      },
      rawResponse: {
        headers: {}
      },
      request: {
        body: JSON.stringify({ prompt: promptText, ...generationOptions })
      }
    };
  }
  // Helper method to extract prompt text from LanguageModelV1Prompt
  getPromptText(prompt) {
    if (typeof prompt === "string") {
      return prompt;
    }
    if (Array.isArray(prompt)) {
      return prompt.map((message) => {
        if (Array.isArray(message.content)) {
          return message.content.filter((part) => "text" in part).map((part) => "text" in part ? part.text : "").join(" ");
        } else if (typeof message.content === "string") {
          return message.content;
        }
        return "";
      }).join("\n");
    }
    return "";
  }
};

// src/hf-transformersjs-chat-language-model.ts
import { pipeline as pipeline2, TextStreamer as TextStreamer2 } from "@huggingface/transformers";
var HFTransformersjsChatLanguageModel = class {
  constructor(modelId, settings, config) {
    this.specificationVersion = "v1";
    this.defaultObjectGenerationMode = "json";
    console.log(`[DEBUG] Initializing HFTransformersjsChatLanguageModel with modelId: ${modelId}`);
    this.modelId = modelId;
    this.settings = settings;
    this.config = config;
    this.provider = config.provider;
  }
  // Lazy-load the text generation pipeline.
  async ensurePipeline() {
    console.log("[DEBUG] ensurePipeline() called");
    if (!this.pipelineInstance) {
      console.log(`[DEBUG] Pipeline not initialized, loading model: ${this.modelId}`);
      try {
        this.pipelineInstance = await pipeline2("text-generation", this.modelId, {
          progress_callback: (progress) => {
            console.log(`[DEBUG] Model loading progress: ${JSON.stringify(progress)}%`);
          }
        });
        console.log(`[DEBUG] Pipeline successfully loaded`);
      } catch (error) {
        console.error(`[ERROR] Failed to load pipeline:`, error);
        throw error;
      }
    } else {
      console.log("[DEBUG] Using existing pipeline instance");
    }
    return this.pipelineInstance;
  }
  doGenerate(options) {
    console.log("[DEBUG] doGenerate() called");
    return this.doGenerateImpl(options);
  }
  async doGenerateImpl(options) {
    var _a, _b, _c;
    console.log("[DEBUG] doGenerateImpl() called with options:", JSON.stringify(options, null, 2));
    console.log("[DEBUG] Getting pipeline...");
    const pn = await this.ensurePipeline();
    console.log("[DEBUG] Pipeline retrieved successfully");
    console.log("[DEBUG] Converting prompt to string");
    const promptText = this.convertPromptToString(options.prompt);
    console.log("[DEBUG] Prompt converted to string:", promptText.substring(0, 100) + (promptText.length > 100 ? "..." : ""));
    const generationOptions = {
      max_new_tokens: (_a = options.maxTokens) != null ? _a : 512,
      do_sample: ((_b = options.temperature) != null ? _b : 0) > 0,
      temperature: (_c = options.temperature) != null ? _c : 0
    };
    console.log("[DEBUG] Generation options:", generationOptions);
    console.log("[DEBUG] Calling pipeline...");
    try {
      const res = await pn(promptText, generationOptions);
      console.log("[DEBUG] Pipeline result received");
      console.log("[DEBUG] Raw pipeline result:", JSON.stringify(res, null, 2));
      const result = Array.isArray(res) ? res : [res];
      console.log("[DEBUG] Pipeline result processed");
      if (!result || !result[0] || !result[0].generated_text === void 0) {
        console.error("[ERROR] Unexpected pipeline output format:", result);
        throw new Error("Unexpected pipeline output format");
      }
      const promptTokens = Math.ceil(promptText.length / 4);
      const generatedText = result[0].generated_text.toString();
      const completionTokens = Math.ceil(generatedText.length / 4);
      console.log("[DEBUG] Generated text length:", generatedText.length);
      console.log("[DEBUG] First 100 chars of generated text:", generatedText.substring(0, 100) + (generatedText.length > 100 ? "..." : ""));
      return {
        text: generatedText,
        finishReason: "stop",
        // Assuming normal completion
        usage: {
          promptTokens,
          completionTokens
        },
        rawCall: {
          rawPrompt: promptText,
          rawSettings: generationOptions
        },
        rawResponse: {
          body: result
        },
        request: {
          body: JSON.stringify({ prompt: promptText, ...generationOptions })
        }
      };
    } catch (error) {
      console.error("[ERROR] Pipeline execution failed:", error);
      throw error;
    }
  }
  // Helper method to convert LanguageModelV1Prompt to string
  convertPromptToString(prompt) {
    console.log("[DEBUG] convertPromptToString() called with prompt type:", typeof prompt);
    if (typeof prompt === "string") {
      console.log("[DEBUG] Prompt is a string");
      return prompt;
    }
    if (Array.isArray(prompt)) {
      console.log("[DEBUG] Prompt is an array of messages with length:", prompt.length);
      return prompt.map((message) => {
        console.log("[DEBUG] Processing message with role:", message.role);
        if (Array.isArray(message.content)) {
          console.log("[DEBUG] Message content is an array of parts with length:", message.content.length);
          const result = message.content.filter((part) => "text" in part).map((part) => part.text).join(" ");
          console.log("[DEBUG] Extracted text from parts:", result.substring(0, 50) + (result.length > 50 ? "..." : ""));
          return result;
        } else if (typeof message.content === "string") {
          console.log("[DEBUG] Message content is a string");
          const result = `${message.role}: ${message.content}`;
          console.log("[DEBUG] Formatted message:", result.substring(0, 50) + (result.length > 50 ? "..." : ""));
          return result;
        }
        console.log("[DEBUG] Unknown message content format");
        return "";
      }).join("\n");
    }
    console.log("[DEBUG] Unrecognized prompt format, returning empty string");
    return "";
  }
  doStream(options) {
    console.log("[DEBUG] doStream() called");
    return this.doStreamImplementation(options);
  }
  async doStreamImplementation(options) {
    var _a, _b, _c;
    console.log("[DEBUG] doStreamImplementation() called");
    console.log("[DEBUG] Getting pipeline for streaming...");
    const pn = await this.ensurePipeline();
    console.log("[DEBUG] Pipeline for streaming retrieved successfully");
    const promptText = this.convertPromptToString(options.prompt);
    console.log("[DEBUG] Prompt converted to string for streaming");
    const generationOptions = {
      max_new_tokens: (_a = options.maxTokens) != null ? _a : 512,
      do_sample: ((_b = options.temperature) != null ? _b : 0) > 0,
      temperature: (_c = options.temperature) != null ? _c : 0
    };
    console.log("[DEBUG] Streaming generation options:", generationOptions);
    const stream = new ReadableStream({
      start: async (controller) => {
        console.log("[DEBUG] Stream start callback executing");
        try {
          const tokenCallback = (token) => {
            if (token) {
              console.log(`[DEBUG] Token received: "${token}"`);
              controller.enqueue({ type: "text-delta", textDelta: token });
            }
          };
          console.log("[DEBUG] Creating TextStreamer");
          const streamer = new TextStreamer2(pn.tokenizer, {
            skip_prompt: true,
            callback_function: tokenCallback
          });
          console.log("[DEBUG] TextStreamer created");
          const streamerOptions = {
            ...generationOptions,
            streamer
          };
          console.log("[DEBUG] Starting streaming generation");
          const result = await pn(promptText, streamerOptions);
          console.log("[DEBUG] Streaming generation completed");
          console.log("[DEBUG] Closing stream controller");
          controller.close();
          return result;
        } catch (error) {
          console.error("[ERROR] Error in stream processing:", error);
          controller.error(error);
        }
      }
    });
    return {
      stream,
      rawCall: {
        rawPrompt: promptText,
        rawSettings: generationOptions
      },
      rawResponse: {
        headers: {}
      },
      request: { body: JSON.stringify({ prompt: promptText, ...generationOptions }) }
    };
  }
};

// src/hf-transformersjs-completion-language-model.ts
import { pipeline as pipeline3, TextStreamer as TextStreamer3 } from "@huggingface/transformers";
var HFTransformersjsCompletionLanguageModel = class {
  constructor(modelId, settings, config) {
    this.specificationVersion = "v1";
    this.defaultObjectGenerationMode = "json";
    this.modelId = modelId;
    this.settings = settings;
    this.config = config;
    this.provider = config.provider;
  }
  async ensurePipeline() {
    if (!this.pipelineInstance) {
      this.pipelineInstance = await pipeline3("text-generation", this.modelId, {});
    }
    return this.pipelineInstance;
  }
  doGenerate(options) {
    return this.doGenerateImpl(options);
  }
  async doGenerateImpl(options) {
    var _a, _b, _c;
    const pn = await this.ensurePipeline();
    const promptText = this.getPromptText(options.prompt);
    const generationOptions = {
      max_new_tokens: (_a = options.maxTokens) != null ? _a : 512,
      do_sample: ((_b = options.temperature) != null ? _b : 0) > 0,
      temperature: (_c = options.temperature) != null ? _c : 0
    };
    const res = await pn(promptText, generationOptions);
    const result = Array.isArray(res) ? res : [res];
    const promptTokens = Math.ceil(promptText.length / 4);
    const completionTokens = Math.ceil(result[0][-1].generated_text.length / 4);
    return {
      text: result[0][-1].generated_text.toString(),
      finishReason: "stop",
      // Assuming normal completion
      usage: {
        promptTokens,
        completionTokens
      },
      rawCall: {
        rawPrompt: promptText,
        rawSettings: generationOptions
      },
      rawResponse: {
        body: result
      },
      request: {
        body: JSON.stringify({ prompt: promptText, ...generationOptions })
      }
    };
  }
  doStream(options) {
    return this.doStreamImpl(options);
  }
  async doStreamImpl(options) {
    var _a, _b, _c;
    const pn = await this.ensurePipeline();
    const promptText = this.getPromptText(options.prompt);
    const generationOptions = {
      max_new_tokens: (_a = options.maxTokens) != null ? _a : 512,
      do_sample: ((_b = options.temperature) != null ? _b : 0) > 0,
      temperature: (_c = options.temperature) != null ? _c : 0
    };
    const stream = new ReadableStream({
      start: async (controller) => {
        try {
          const tokenCallback = (token) => {
            if (token) {
              controller.enqueue({
                type: "text-delta",
                textDelta: token
              });
            }
          };
          const streamer = new TextStreamer3(pn.tokenizer, {
            skip_prompt: true,
            callback_function: tokenCallback
          });
          const streamerOptions = {
            ...generationOptions,
            streamer
          };
          const result = await pn(promptText, streamerOptions);
          controller.enqueue({
            type: "finish"
          });
          controller.close();
          return result;
        } catch (error) {
          controller.error(error);
        }
      }
    });
    return {
      stream,
      rawCall: {
        rawPrompt: promptText,
        rawSettings: generationOptions
      },
      rawResponse: {
        headers: {}
      },
      request: {
        body: JSON.stringify({ prompt: promptText, ...generationOptions })
      }
    };
  }
  // Helper method to extract prompt text from LanguageModelV1Prompt
  getPromptText(prompt) {
    if (typeof prompt === "string") {
      return prompt;
    }
    if (Array.isArray(prompt)) {
      return prompt.map((message) => {
        if (Array.isArray(message.content)) {
          return message.content.filter((part) => "text" in part).map((part) => "text" in part ? part.text : "").join(" ");
        } else if (typeof message.content === "string") {
          return message.content;
        }
        return "";
      }).join("\n");
    }
    return "";
  }
};

// src/hf-transformersjs-embedding-model.ts
import { pipeline as pipeline4 } from "@huggingface/transformers";
import { TooManyEmbeddingValuesForCallError } from "@ai-sdk/provider";
var HFTransformersjsEmbeddingModel = class {
  constructor(modelId, settings, config) {
    this.specificationVersion = "v1";
    this.supportsParallelCalls = true;
    this.modelId = modelId;
    this.settings = settings;
    this.config = config;
    this.provider = config.provider;
  }
  // Define maxEmbeddingsPerCall or use a default:
  get maxEmbeddingsPerCall() {
    return 2048;
  }
  async ensurePipeline() {
    if (!this.pipelineInstance) {
      this.pipelineInstance = await pipeline4("feature-extraction", this.modelId, {
        revision: "main"
        // quantized: false
      });
    }
    return this.pipelineInstance;
  }
  // Helper function to safely extract numeric arrays from pipeline outputs
  extractVectorsFromOutput(output) {
    if (!output) return [[]];
    try {
      if (output && typeof output === "object" && "data" in output) {
        const data = output.data;
        if (Array.isArray(data)) {
          if (data.length === 0) return [[]];
          return Array.isArray(data[0]) ? data : [data];
        }
      }
      if (Array.isArray(output)) {
        if (output.length === 0) return [[]];
        if (Array.isArray(output[0]) && output[0].length > 0 && typeof output[0][0] === "number") {
          return output;
        }
        if (typeof output[0] === "number") {
          return [output];
        }
      }
      if (output && typeof output === "object" && "toArray" in output && typeof output.toArray === "function") {
        const arrayData = output.toArray();
        if (Array.isArray(arrayData)) {
          if (arrayData.length === 0) return [[]];
          return Array.isArray(arrayData[0]) ? arrayData : [arrayData];
        }
      }
      if (output && typeof output === "object" && "length" in output) {
        try {
          const arrayData = Array.from(output);
          if (arrayData.length === 0) return [[]];
          return Array.isArray(arrayData[0]) ? arrayData : [arrayData];
        } catch (e) {
          console.error("Failed to convert array-like object:", e);
        }
      }
      if (output && typeof output === "object") {
        for (const key of ["embedding", "embeddings", "vector", "vectors", "features"]) {
          if (key in output && Array.isArray(output[key])) {
            const vectorData = output[key];
            if (vectorData.length === 0) return [[]];
            return Array.isArray(vectorData[0]) ? vectorData : [vectorData];
          }
        }
      }
      console.warn("Using fallback extraction method for pipeline output:", output);
      return [[]];
    } catch (error) {
      console.error("Error extracting vectors from pipeline output:", error);
      return [[]];
    }
  }
  async doEmbed(options) {
    if (options.values.length > this.maxEmbeddingsPerCall) {
      throw new TooManyEmbeddingValuesForCallError({
        provider: this.config.provider,
        modelId: this.modelId,
        maxEmbeddingsPerCall: this.maxEmbeddingsPerCall,
        values: options.values
      });
    }
    const extractor = await this.ensurePipeline();
    const embeddings = [];
    for (const input of options.values) {
      try {
        const output = await extractor(input, {});
        const tokenVectors = this.extractVectorsFromOutput(output);
        const numTokens = tokenVectors.length;
        if (numTokens === 0 || numTokens === 1 && tokenVectors[0].length === 0) {
          embeddings.push([]);
          continue;
        }
        const dim = tokenVectors[0].length;
        const avg = new Array(dim).fill(0);
        for (const vec of tokenVectors) {
          if (vec && vec.length === dim) {
            for (let i = 0; i < dim; i++) {
              avg[i] += vec[i];
            }
          }
        }
        for (let i = 0; i < dim; i++) {
          avg[i] /= numTokens;
        }
        embeddings.push(avg);
      } catch (error) {
        console.error(`Error embedding input "${input.substring(0, 50)}...":`, error);
        embeddings.push([]);
      }
    }
    return {
      embeddings,
      usage: { tokens: options.values.length },
      rawResponse: { headers: options.headers }
    };
  }
};

// src/hf-transformersjs-provider.ts
function createHFTransformersjs(options) {
  const getCommonModelConfig = () => ({
    provider: options.name,
    apiKey: options.apiKey
  });
  const createChatModel = (modelId, settings = {}) => new HFTransformersjsChatLanguageModel(modelId, settings, getCommonModelConfig());
  const createLanguageModel = (modelId, settings = {}) => createChatModel(modelId, settings);
  const createCompletionModel = (modelId, settings = {}) => new HFTransformersjsCompletionLanguageModel(modelId, settings, getCommonModelConfig());
  const createEmbeddingModel = (modelId, settings = {}) => new HFTransformersjsEmbeddingModel(modelId, settings, getCommonModelConfig());
  const provider = function(modelId, settings) {
    return createLanguageModel(modelId, settings);
  };
  provider.languageModel = createLanguageModel;
  provider.chatModel = createChatModel;
  provider.completionModel = createCompletionModel;
  provider.textEmbeddingModel = createEmbeddingModel;
  return provider;
}

// src/hf-transformersjs-chat-language-model-openai-compatible.ts
import { pipeline as pipeline5, TextStreamer as TextStreamer4 } from "@huggingface/transformers";
var HFTransformersjsChatLanguageModelOpenAICompatible = class {
  constructor(modelId, settings, config) {
    this.specificationVersion = "v1";
    this.defaultObjectGenerationMode = "json";
    this.modelId = modelId;
    this.settings = settings;
    this.config = config;
    this.provider = config.provider;
  }
  async ensurePipeline() {
    if (!this.pipelineInstance) {
      this.pipelineInstance = await pipeline5("text-generation", this.modelId, {});
    }
    return this.pipelineInstance;
  }
  doGenerate(options) {
    return this.doGenerateImpl(options);
  }
  async doGenerateImpl(options) {
    var _a, _b, _c;
    const pn = await this.ensurePipeline();
    const promptText = this.convertPromptToString(options.prompt);
    const generationOptions = {
      max_new_tokens: (_a = options.maxTokens) != null ? _a : 512,
      do_sample: ((_b = options.temperature) != null ? _b : 0) > 0,
      temperature: (_c = options.temperature) != null ? _c : 0
    };
    const res = await pn(promptText, generationOptions);
    const result = Array.isArray(res) ? res : [res];
    const promptTokens = Math.ceil(promptText.length / 4);
    const completionTokens = Math.ceil(result[0][-1].generated_text.toString().length / 4);
    return {
      text: result[0][-1].generated_text.toString(),
      finishReason: "stop",
      usage: {
        promptTokens,
        completionTokens
      },
      rawCall: {
        rawPrompt: promptText,
        rawSettings: generationOptions
      },
      rawResponse: {
        body: result
      },
      request: {
        body: JSON.stringify({ prompt: promptText, ...generationOptions })
      }
    };
  }
  doStream(options) {
    return this.doStreamImplementation(options);
  }
  async doStreamImplementation(options) {
    var _a, _b, _c;
    const pn = await this.ensurePipeline();
    const promptText = this.convertPromptToString(options.prompt);
    const generationOptions = {
      max_new_tokens: (_a = options.maxTokens) != null ? _a : 512,
      do_sample: ((_b = options.temperature) != null ? _b : 0) > 0,
      temperature: (_c = options.temperature) != null ? _c : 0
    };
    const stream = new ReadableStream({
      start: async (controller) => {
        try {
          const tokenCallback = (token) => {
            if (token) {
              controller.enqueue({ type: "text-delta", textDelta: token });
            }
          };
          const streamer = new TextStreamer4(pn.tokenizer, {
            skip_prompt: true,
            callback_function: tokenCallback
          });
          const streamerOptions = {
            ...generationOptions,
            streamer
          };
          const result = await pn(promptText, streamerOptions);
          controller.close();
          return result;
        } catch (error) {
          controller.error(error);
        }
      }
    });
    return {
      stream,
      rawCall: {
        rawPrompt: promptText,
        rawSettings: generationOptions
      },
      rawResponse: {
        headers: {}
      },
      request: { body: JSON.stringify({ prompt: promptText, ...generationOptions }) }
    };
  }
  // Helper method to convert LanguageModelV1Prompt to string
  convertPromptToString(prompt) {
    if (typeof prompt === "string") {
      return prompt;
    }
    if (Array.isArray(prompt)) {
      return prompt.map((message) => {
        if (Array.isArray(message.content)) {
          return message.content.filter((part) => "text" in part).map((part) => part.text).join(" ");
        } else if (typeof message.content === "string") {
          return `${message.role}: ${message.content}`;
        }
        return "";
      }).join("\n");
    }
    return "";
  }
};
export {
  HFTransformersjsChatLanguageModel,
  HFTransformersjsChatLanguageModelOpenAICompatible,
  HFTransformersjsCompletionLanguageModel,
  HFTransformersjsCompletionLanguageModelOpenAICompatible,
  HFTransformersjsEmbeddingModel,
  createHFTransformersjs
};
//# sourceMappingURL=index.mjs.map